"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http_1 = require("@nativescript/core/http");
var http_request_1 = require("@nativescript/core/http/http-request");
var image_source_1 = require("@nativescript/core/image-source/image-source");
var http_common_1 = require("./http.common");
var fs = require("tns-core-modules/file-system");
var platform_1 = require("tns-core-modules/platform");
var debugger_1 = require("tns-core-modules/debugger");
var http_common_2 = require("./http.common");
exports.HTTPFormData = http_common_2.HTTPFormData;
exports.HTTPFormDataEntry = http_common_2.HTTPFormDataEntry;
function parseJSON(source) {
    var src = source.trim();
    if (src.lastIndexOf(")") === src.length - 1) {
        return JSON.parse(src.substring(src.indexOf("(") + 1, src.lastIndexOf(")")));
    }
    return JSON.parse(src);
}
var requestIdCounter = 0;
var pendingRequests = {};
var completeCallback;
function ensureCompleteCallback() {
    if (completeCallback) {
        return;
    }
    completeCallback = new com.klippa.NativeScriptHTTP.Async.CompleteCallback({
        onComplete: function (result, context) {
            onRequestComplete(context, result);
        },
        onError: function (error, context) {
            onRequestError(error, context);
        },
    });
}
function onRequestComplete(requestId, result) {
    var callbacks = pendingRequests[requestId];
    delete pendingRequests[requestId];
    if (result.error) {
        callbacks.rejectCallback(new Error(result.error.toString()));
        return;
    }
    var headers = {};
    if (result.headers) {
        var jHeaders = result.headers;
        var length_1 = jHeaders.size();
        var pair = void 0;
        for (var i = 0; i < length_1; i++) {
            pair = jHeaders.get(i);
            http_request_1.addHeader(headers, pair.key, pair.value);
        }
    }
    if (global.__inspector && global.__inspector.isConnected) {
        debugger_1.NetworkAgent.responseReceived("okhttp3_" + requestId, result, headers);
    }
    callbacks.resolveCallback({
        content: {
            raw: result.raw,
            toArrayBuffer: function () { return Uint8Array.from(result.raw.toByteArray()).buffer; },
            toString: function (encoding) {
                var str;
                if (encoding) {
                    str = decodeResponse(result.raw, encoding);
                }
                else {
                    str = result.responseAsString;
                }
                if (typeof str === "string") {
                    return str;
                }
                else {
                    throw new Error("Response content may not be converted to string");
                }
            },
            toJSON: function (encoding) {
                var str;
                if (encoding) {
                    str = decodeResponse(result.raw, encoding);
                }
                else {
                    str = result.responseAsString;
                }
                return parseJSON(str);
            },
            toImage: function () {
                return new Promise(function (resolveImage, rejectImage) {
                    if (result.responseAsImage != null) {
                        resolveImage(new image_source_1.ImageSource(result.responseAsImage));
                    }
                    else {
                        rejectImage(new Error("Response content may not be converted to an Image"));
                    }
                });
            },
            toFile: function (destinationFilePath) {
                if (!destinationFilePath) {
                    destinationFilePath = http_common_1.getFilenameFromUrl(callbacks.url);
                }
                var stream;
                try {
                    var file = fs.File.fromPath(destinationFilePath);
                    var javaFile = new java.io.File(destinationFilePath);
                    stream = new java.io.FileOutputStream(javaFile);
                    stream.write(result.raw.toByteArray());
                    return file;
                }
                catch (exception) {
                    throw new Error("Cannot save file with path: " + destinationFilePath + ".");
                }
                finally {
                    if (stream) {
                        stream.close();
                    }
                }
            }
        },
        statusCode: result.statusCode,
        headers: headers
    });
}
function onRequestError(error, requestId) {
    var callbacks = pendingRequests[requestId];
    delete pendingRequests[requestId];
    if (callbacks) {
        callbacks.rejectCallback(new Error(error));
    }
}
function buildJavaOptions(options) {
    if (typeof options.url !== "string") {
        throw new Error("Http request must provide a valid url.");
    }
    var javaOptions = new com.klippa.NativeScriptHTTP.Async.Http.RequestOptions();
    javaOptions.url = options.url;
    var contentType;
    var mediaType = null;
    if (options.headers) {
        for (var key in options.headers) {
            if (key.toLowerCase() === "content-type") {
                contentType = options.headers[key];
            }
        }
        if (contentType) {
            mediaType = okhttp3.MediaType.parse(contentType);
        }
    }
    if (typeof options.method === "string") {
        javaOptions.method = options.method;
    }
    if (options.content instanceof FormData || options.content instanceof http_common_1.HTTPFormData) {
        var builder_1 = new okhttp3.MultipartBody.Builder();
        builder_1.setType(okhttp3.MediaType.parse("multipart/form-data"));
        options.content.forEach((function (value, key) {
            if (typeof (value) === "string") {
                builder_1.addFormDataPart(key, value);
            }
            else if (value != null && typeof (value) !== "undefined" && !(value instanceof http_common_1.HTTPFormDataEntry)) {
                var MEDIA_TYPE = okhttp3.MediaType.parse(value.type);
                var typedArray = new Uint8Array(Blob.InternalAccessor.getBuffer(value));
                var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
                builder_1.addFormDataPart(key, value.name, okhttp3.RequestBody.create(nativeBuffer, MEDIA_TYPE));
            }
            else if (value instanceof http_common_1.HTTPFormDataEntry) {
                var MEDIA_TYPE = okhttp3.MediaType.parse(value.contentType);
                if (value.data instanceof ArrayBuffer) {
                    var typedArray = new Uint8Array(value.data);
                    var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
                    value.data = okhttp3.RequestBody.create(nativeBuffer, mediaType);
                }
                builder_1.addFormDataPart(key, value.fileName, okhttp3.RequestBody.create(value.data, MEDIA_TYPE));
            }
        }));
        javaOptions.content = builder_1.build();
    }
    else if (typeof options.content === "string") {
        var nativeString = new java.lang.String(options.content.toString());
        javaOptions.content = okhttp3.RequestBody.create(nativeString, mediaType);
    }
    else if (options.content instanceof ArrayBuffer) {
        var typedArray = new Uint8Array(options.content);
        var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
        javaOptions.content = okhttp3.RequestBody.create(nativeBuffer, mediaType);
    }
    if (typeof options.timeout === "number") {
        javaOptions.timeout = options.timeout;
    }
    if (typeof options.dontFollowRedirects === "boolean") {
        javaOptions.dontFollowRedirects = options.dontFollowRedirects;
    }
    if (options.headers) {
        var arrayList = new java.util.ArrayList();
        var pair = com.klippa.NativeScriptHTTP.Async.Http.KeyValuePair;
        for (var key in options.headers) {
            arrayList.add(new pair(key, options.headers[key] + ""));
        }
        javaOptions.headers = arrayList;
    }
    var ourScreen = platform_1.screen.mainScreen;
    javaOptions.screenWidth = ourScreen.widthPixels;
    javaOptions.screenHeight = ourScreen.heightPixels;
    return javaOptions;
}
function request(options) {
    if (options === undefined || options === null) {
        return Promise.reject("No options given");
    }
    return new Promise(function (resolve, reject) {
        try {
            var javaOptions = buildJavaOptions(options);
            if (global.__inspector && global.__inspector.isConnected) {
                debugger_1.NetworkAgent.requestWillBeSent("okhttp3_" + requestIdCounter, options);
            }
            var callbacks = {
                url: options.url,
                resolveCallback: resolve,
                rejectCallback: reject
            };
            pendingRequests[requestIdCounter] = callbacks;
            ensureCompleteCallback();
            com.klippa.NativeScriptHTTP.Async.Http.MakeRequest(javaOptions, completeCallback, new java.lang.Integer(requestIdCounter));
            requestIdCounter++;
        }
        catch (ex) {
            reject(ex);
        }
    });
}
exports.request = request;
function decodeResponse(raw, encoding) {
    var charsetName = "UTF-8";
    if (encoding === http_1.HttpResponseEncoding.GBK) {
        charsetName = "GBK";
    }
    return raw.toString(charsetName);
}
function getString(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var str = r.content.toString();
                resolve(str);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getString = getString;
function getJSON(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var json = r.content.toJSON();
                resolve(json);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getJSON = getJSON;
function getImage(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                resolve(r.content.toImage());
            }
            catch (err) {
                reject(err);
            }
        }, function (err) {
            reject(err);
        });
    });
}
exports.getImage = getImage;
function getFile(arg, destinationFilePath) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var file = r.content.toFile(destinationFilePath);
                resolve(file);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getFile = getFile;
function getBinary(arg) {
    return new Promise(function (resolve, reject) {
        request(typeof arg === "string" ? { url: arg, method: "GET" } : arg)
            .then(function (r) {
            try {
                var arrayBuffer = r.content.toArrayBuffer();
                resolve(arrayBuffer);
            }
            catch (e) {
                reject(e);
            }
        }, function (e) { return reject(e); });
    });
}
exports.getBinary = getBinary;
//# sourceMappingURL=http.android.js.map